import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.*;
import java.sql.*;
import java.beans.*;
import java.text.*;

/**
 * This panel shows the average and trend times.
 *
 * @author tjacopi
 *
 */
public class AveragePanel extends JPanel implements PropertyChangeListener {

    protected SegmentedData       data = null;
    protected JLabel              avgLabel = null;
    protected QStatusPanel        statusPanel = null;
    protected String              avgText = null;
    protected int                 numTrendValues = 0;
    protected int                 noDataReceivedCount = 0;
    protected DecimalFormat       decimalFormat = new DecimalFormat("##,##0.0");
    protected ImageIcon trendDecreasing = null;
    protected ImageIcon trendIncreasing = null;
    protected ImageIcon trendNeutral    = null;

    public AveragePanel(SegmentedData data, String avgText, ImageIcon trendIncreasing,
                        ImageIcon trendDecreasing, ImageIcon trendNeutral, int numTrendValues, boolean showMsgs, String noDataMsg) {
      this.data = data;
      this.avgText = avgText;
      this.trendIncreasing = trendIncreasing;
      this.trendDecreasing = trendDecreasing;
      this.trendNeutral    = trendNeutral;
      this.numTrendValues  = numTrendValues;

      setLayout(new BorderLayout() );
      setBorder( new EmptyBorder(0,3,0,3) );
      setBackground(Color.black);
      avgLabel = new JLabel(avgText);
      avgLabel.setHorizontalTextPosition(SwingConstants.LEFT);
      avgLabel.setBackground(Color.black);
      avgLabel.setForeground(Color.yellow);
      add(avgLabel, "West");


      JPanel blankPanel = new JPanel();
      blankPanel.setBackground(Color.black);
      blankPanel.setBorder( new EmptyBorder(0,12,0,12) );
      add(blankPanel, "Center");

      if (showMsgs) {
        statusPanel = new QStatusPanel(noDataMsg);
        add(statusPanel, "East");
      };

      data.addPropertyChangeListener(this);
      calcValues();

    }

   public void reset() {
     calcValues();
   }

	/**
    * This is called whenever the timer pops.
	 */
    public void propertyChange(PropertyChangeEvent e) {
      calcValues();
    }

    protected void calcValues() {
      int numValues = data.getNumValues();
      double avgValue = 0.0;
      if (numValues > 0) {
        if (numValues > 1000) {
          numValues = 1000;
        };
        avgValue = data.getAverageValue(numValues);

        if (numTrendValues>0) {
          double trendAvgValue = data.getAverageValue(numTrendValues);
          ImageIcon icon = trendNeutral;
          if (trendAvgValue>avgValue) {
            icon = trendIncreasing;
          } else if (trendAvgValue<avgValue) {
            icon = trendDecreasing;
          }
          avgLabel.setIcon(icon);
        }
      } else {
        if (numTrendValues>0) {
          avgLabel.setIcon(trendNeutral);
        }
      }

      String txt = decimalFormat.format( avgValue );
      Object[] parms = new Object[1];
      parms[0] = txt;
      String msgText = MessageFormat.format(avgText, parms);
      avgLabel.setText(msgText);

      if (statusPanel   != null) {                                // Should we show no activity label?
        if (data.dataReceivedForLastInterval() ) {
          noDataReceivedCount = 0;
        } else {
          noDataReceivedCount++;
        }

        if (noDataReceivedCount>1 && (statusPanel.getNoDataMsg() != null) ) {
          System.out.println("Attempting to show NoDataMsg");
          statusPanel.show(QStatusPanel.NoDataMsg);
        } else {
          if (data.getNumValues() > 0 && data.isLatestValueNonZero() ) {        // Anything nonzero is activity
            statusPanel.show(QStatusPanel.NoMsg);
          } else {
            statusPanel.show(QStatusPanel.NoActivityMsg);
          }
        }
      };
    }


}