import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.*;
import java.sql.*;

/**
 * The CaptureDataGatherer processes the data returned by the queries and stores the results in
 * a set of hashtables.  The parent DataGatherer class then looks at the hashtables and processes
 * the required notifications.
 *
 * @author tjacopi
 *
 */
public class CaptureDataGatherer extends DataGatherer {

    public CaptureDataGatherer(String schema) {
      super(schema);
    }


	/**
    * This is called to process the result set.
	 */
    protected Timestamp getValues(int queryNumber, Timestamp previousLatestMonitorTime, ResultSet rs, Hashtable latencyValues, Hashtable latencyRowCounts, Hashtable tpValues, Hashtable latestTimes) {
      Timestamp latestMonitorTime = previousLatestMonitorTime;
      String latencyKey = CaptureInstanceMonitorPanel.AllCaptureLogs;
      String tpKey = CaptureInstanceMonitorPanel.AllCaptureTP;

      try {
        while (rs.next() ) {
          Timestamp  time = rs.getTimestamp(1);
          Timestamp  logTime = rs.getTimestamp(2);
          int rows = rs.getInt(3);

          if (time.after(latestMonitorTime) ) {
            latestMonitorTime = (Timestamp) time.clone();        // Save the latest timestamp for next WHERE clause
          };

          // Fill out the latency values
          double latency = (double) (time.getTime() - logTime.getTime());
//        System.out.println("fetched latency = " + latency + " with rows applied : " + rows );

          // Now keep latency total by queue
          if (latency > 0 ) {                            // Do not let no rows replicated influence latency
            int[] latencyRowCount = (int[]) latencyRowCounts.get(latencyKey);
            if (latencyRowCount == null ) {
              latencyRowCount = new int[1];
              latencyRowCount[0] = 0;
              latencyRowCounts.put(latencyKey, latencyRowCount);
            };
            latencyRowCount[0] = latencyRowCount[0] + 1;
          };

          double[] latencyValue = (double[]) latencyValues.get(latencyKey);
          if (latencyValue == null ) {
            latencyValue = new double[1];
            latencyValue[0] = 0.0;
            latencyValues.put(latencyKey, latencyValue);
          };
          latencyValue[0] = latencyValue[0] + latency;

          Timestamp latestTimeForSubset = (Timestamp) latestTimes.get(tpKey);
          if (latestTimeForSubset == null || time.after(latestTimeForSubset) ) {
            latestTimes.put(tpKey, time.clone() );
          }

          // Keep sum of total rows by queue
          double[] tpValue = (double[]) tpValues.get(tpKey);
          if (tpValue == null ) {
            tpValue = new double[1];
            tpValue[0] = 0.0;
            tpValues.put(tpKey, tpValue);
          };
          tpValue[0] = tpValue[0] + rows;
        }
      } catch( Exception e ) {
        System.out.println(e);
        e.printStackTrace();
      }

      return latestMonitorTime;
    }


   protected String[] getMonitorQueries() {
     String[] sqlStatements = new String[1];
     sqlStatements[0] = "SELECT  MONITOR_TIME, SYNCHTIME, CD_ROWS_INSERTED FROM " +
                        schema+".IBMSNAP_CAPMON  WHERE MONITOR_TIME> ? " +
                        " ORDER BY  MONITOR_TIME DESC" ;
     return sqlStatements;
   }

   protected String getRefreshIntervalQuery() {
     return  "SELECT MONITOR_INTERVAL, '0801' FROM " + schema + ".IBMSNAP_CAPPARMS ";
   }

}