import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;


/**
 * The main class for the CaptureMonitor
 *
 * @author tjacopi
 *
 */
public class CaptureMonitor extends MonitorBase  {


  public static void main(String args[]) {

    init(args);

    JFrame monitor = createMonitor(aAlias, aSchema, aUid, aPwd, new Boolean(true) );
    if (monitor == null) {
        System.exit(0);
    };

  }


  public static JFrame createMonitor(String alias, String schema, String uid, String pwd, Boolean bExitOnClose) {
    MonitorFrame frame = null;

    if ( alias != null && schema != null && uid != null && pwd != null ) {

      JFrame parent = null;   // How to get the desktop?
      Hashtable savedData = readSavedData();
      ReplInstanceMonitorPanel monitorPanel = new CaptureInstanceMonitorPanel(alias, uid, pwd, schema);
      monitorPanel.addMonitoring( ReplInstanceMonitorPanel.LatencyGraph, CaptureInstanceMonitorPanel.AllCaptureLogs );
      monitorPanel.addMonitoring( ReplInstanceMonitorPanel.ThroughputGraph, CaptureInstanceMonitorPanel.AllCaptureTP );
//    monitorPanel.addMonitoring( ReplInstanceMonitorPanel.ThroughputGraph, ReplInstanceMonitorPanel.AllQueues );

      String title = "Capture Monitor - " + alias + ":" + schema;
      frame = new MonitorFrame(title, monitorPanel);

      String txt = "The refresh interval is determined by the MONITOR_INTERVAL column in the IBMSNAP_CAPPARMS table.\n" +
          "To change the refresh time, update the MONITOR_INTERVAL column and then select the \n" +
          "File->Stop and File->Start actions. \n\n" +
          "Note that the MONITIOR_INTERVAL value is in seconds, so a value of 5 to 10 works good.";
      frame.setRefreshMsg(txt);
      frame.setHelpFile("CaptureHelp.htm");

      frame.pack();
      if (useProfile) {
        frame.loadInformation(savedData);           // Let the frame set its size/position
      };
      frame.setVisible(true);

      monitorPanel.start();
      addShutdownListener( frame, bExitOnClose.booleanValue() );

    } else {
      printHelp();
    }
    return frame;
  }

  protected static void printHelp() {
    System.out.println("Usage:  CaptureMonitor -alias <Capture alias> -schema <Capture schema> -uid <userid> -pwd <password>");
    printParams();
    System.out.println("  example:  CaptureMonitor -alias RED -schema ASN -uid tjacopi -pwd hiThere1");
  }

}