import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.*;
import java.sql.*;

/**
 * This is the subclass that gathers data specifically for QApply.
 *
 * @author tjacopi
 *
 */
public class QApplyDataGatherer extends DataGatherer {

    public QApplyDataGatherer(String schema) {
      super(schema);
    }


	/**
    * This is called to process the result set.
	 */
    protected Timestamp getValues(int queryNumber, Timestamp previousLatestMonitorTime, ResultSet rs, Hashtable latencyValues, Hashtable latencyRowCounts, Hashtable tpValues, Hashtable latestTimes) {
      Timestamp latestMonitorTime = previousLatestMonitorTime;
      try {
        while (rs.next() ) {
          String queue = rs.getString(2);
          Timestamp  time = rs.getTimestamp(1);

          if (time.after(latestMonitorTime) ) {
            latestMonitorTime = (Timestamp) time.clone();        // Save the latest timestamp for next WHERE clause
          };

          Timestamp latestTimeForQueue = (Timestamp) latestTimes.get(queue);
          if (latestTimeForQueue == null || time.after(latestTimeForQueue) ) {
            latestTimes.put(queue, time.clone() );
          }

          int qLatency = rs.getInt(3);
          int appLatency = rs.getInt(4);
          int capLatency = rs.getInt(5);
          int rowsApplied = rs.getInt(6);
          int rowsNotApplied = rs.getInt(7);

          // Keep sum of total rows by queue
          double[] tpValue = (double[]) tpValues.get(queue);
          if (tpValue == null ) {
            tpValue = new double[2];
            tpValue[0] = 0.0;
            tpValue[1] = 0.0;
            tpValues.put(queue, tpValue);
          };
          tpValue[0] = tpValue[0] + rowsApplied;

          // This is nonzero if we had an exception.
          if (rowsNotApplied >0) {
            tpValue[1] = tpValue[1] + rowsNotApplied;
          };

          // Now keep latency total by queue
          if (qLatency  > 0 || appLatency > 0 || capLatency > 0) {       // Do not let no rows replicated influence latency
            int[] latencyRowCount = (int[]) latencyRowCounts.get(queue);
            if (latencyRowCount == null ) {
              latencyRowCount = new int[1];
              latencyRowCount[0] = 0;
              latencyRowCounts.put(queue, latencyRowCount);
            };
            latencyRowCount[0] = latencyRowCount[0] + 1;
          };

          double[] latencyValue = (double[]) latencyValues.get(queue);
          if (latencyValue == null ) {
            latencyValue = new double[3];
            latencyValue[0] = 0.0;
            latencyValue[1] = 0.0;
            latencyValue[2] = 0.0;
            latencyValues.put(queue, latencyValue);
          };
          latencyValue[0] = latencyValue[0] + appLatency;
          latencyValue[1] = latencyValue[1] + qLatency;
          latencyValue[2] = latencyValue[2] + capLatency;
        }

      } catch( Exception e ) {
        System.out.println(e);
        e.printStackTrace();
      }

      return latestMonitorTime;
    }

   protected String[] getMonitorQueries() {
     String[] sqlStatements = new String[1];
     sqlStatements[0] = "SELECT  MONITOR_TIME, RECVQ, QLATENCY, APPLY_LATENCY, " +
                        " (END2END_LATENCY-QLATENCY-APPLY_LATENCY), ROWS_APPLIED, ROWS_NOT_APPLIED  FROM " + schema+".IBMQREP_APPLYMON  WHERE MONITOR_TIME> ? " +
                        " ORDER BY  MONITOR_TIME DESC" ;
     return sqlStatements;
   }

   protected String getRefreshIntervalQuery() {
     return  "SELECT MONITOR_INTERVAL, ARCH_LEVEL FROM " + schema + ".IBMQREP_APPLYPARMS ";
   }

}