import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;


/**
 * This is the starting class for the QApply monitor.
 *
 * @author tjacopi
 *
 */
public class QApplyMonitor extends MonitorBase  {


  public static void main(String args[]) {

    init(args);

    JFrame monitor = createMonitor(aAlias, aSchema, aUid, aPwd, new Boolean(true) );
    if (monitor == null) {
        System.exit(0);
    };

  }


  public static JFrame createMonitor(String alias, String schema, String uid, String pwd, Boolean bExitOnClose) {
    MonitorFrame frame = null;

    if ( alias != null && schema != null && uid != null && pwd != null ) {

      JFrame parent = null;   // How to get the desktop?
      Vector allQueues = QApplyInstanceMonitorPanel.getRecvQList(alias, schema, uid,  pwd);
      Hashtable savedData = readSavedData();
      Object[] initialQueues = null;
      String title = "QApply Monitor - " + alias + ":" + schema;
      if (allQueues.size() == 0) {
        JOptionPane.showMessageDialog(null, "There are no queues defined in the IBMQREP_RECVQUEUES table. \nThis monitor is based on queues, so you need to define the queues \nfirst and then restart the monitor.", "Error", JOptionPane.ERROR_MESSAGE);
      } else if (allQueues.size() == 1) {
        initialQueues = new String[1];
        initialQueues[0] = allQueues.elementAt(0).toString();
      } else {

        // Since we have more than one possible queue, try to use the queues last used
        String savedQueuesKey = MonitorFrame.generateQueuesSaveKey(title);
        Vector savedQueues = (Vector) savedData.get(savedQueuesKey);

        if (useProfile && savedQueues != null && savedQueues.size()>0 ) {
          savedQueues.retainAll(allQueues);
        } else {
          savedQueues = null;
        }

        // If we do not have any matching saved queues, prompt the user
        if (savedQueues == null || savedQueues.size() == 0 ) {
          String dlgTitle = "Add Queue - " + alias + ":" + schema;
          QueueSelectionDialog selDialog = new QueueSelectionDialog(parent, dlgTitle, true, allQueues);
          selDialog.pack();
          centerWindow(selDialog);
          selDialog.show();
          if (selDialog.wasOkPressed()) {
           initialQueues = selDialog.getSelectedQueues();
          };
        } else {
          initialQueues = new String[savedQueues.size()];
          savedQueues.copyInto(initialQueues);
        }
      }
      if (initialQueues != null) {
        ReplInstanceMonitorPanel monitorPanel = new QApplyInstanceMonitorPanel(alias, uid, pwd, schema);
        for (int i=0; i<initialQueues.length; i++) {
          monitorPanel.addMonitoring( (ReplInstanceMonitorPanel.LatencyGraph | ReplInstanceMonitorPanel.ThroughputGraph), initialQueues[i].toString() );
        }

        frame = new MonitorFrame(title, monitorPanel);

        String txt = "The refresh interval is determined by the MONITOR_INTERVAL column in the IBMQREP_APPPARMS table.\n" +
          "To change the refresh time, update the MONITOR_INTERVAL column and then select the \n" +
          "File->Stop and File->Start actions. \n\n" +
          "Note that the MONITIOR_INTERVAL value is in seconds, so a value of 5 to 10 works good.";
        frame.setRefreshMsg(txt);
        frame.setHelpFile("QApplyHelp.htm");

	     frame.pack();
        if (useProfile) {
          frame.loadInformation(savedData);           // Let the frame set its size/position
        };
	     frame.setVisible(true);

        monitorPanel.start();
        addShutdownListener( frame, bExitOnClose.booleanValue() );
      }

    } else {
      printHelp();
    }

    return frame;
  }



  protected static void printHelp() {
    System.out.println("Usage:  QApplyMonitor -alias <QApply alias> -schema <QApply schema> -uid <userid> -pwd <password>");
    printParams();
    System.out.println("  example:  QApplyMonitor -alias RED -schema ASN -uid tjacopi -pwd hiThere1");
  }

}