import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.*;
import java.sql.*;

/**
 * This is the specific data gathering info for QCapture.
 *
 * @author tjacopi
 *
 */
public class QCaptureDataGatherer extends DataGatherer {

    public QCaptureDataGatherer(String schema) {
      super(schema);
    }


	/**
    * This is called to process the result set.
	 */
    protected Timestamp getValues(int queryNumber, Timestamp previousLatestMonitorTime, ResultSet rs, Hashtable latencyValues, Hashtable latencyRowCounts, Hashtable tpValues, Hashtable latestTimes) {
      Timestamp latestTime = previousLatestMonitorTime;
      if (queryNumber == 0) {
        latestTime = getLatencyValues(previousLatestMonitorTime, rs, latencyValues, latencyRowCounts);
      } else if (queryNumber == 1) {
        latestTime = getTPValues(previousLatestMonitorTime, rs, tpValues, latestTimes);
      }

      return latestTime;
    }

	/**
    * This is called to process the result set.
	 */
    protected Timestamp getLatencyValues(Timestamp previousLatestMonitorTime, ResultSet rs, Hashtable latencyValues, Hashtable latencyRowCounts) {
      Timestamp latestMonitorTime = previousLatestMonitorTime;
      String queue = ReplInstanceMonitorPanel.AllQueues;
      try {
        if (rs.next() ) {
          Timestamp  time = rs.getTimestamp(1);
          Timestamp  logTime = rs.getTimestamp(2);

          if (time.after(latestMonitorTime) ) {
            latestMonitorTime = (Timestamp) time.clone();        // Save the latest timestamp for next WHERE clause
          };

          double latency = (double) (time.getTime() - logTime.getTime());

          // Now keep latency total by queue
          if (latency > 0 ) {                            // Do not let no rows replicated influence latency
            int[] latencyRowCount = (int[]) latencyRowCounts.get(queue);
            if (latencyRowCount == null ) {
              latencyRowCount = new int[1];
              latencyRowCount[0] = 0;
              latencyRowCounts.put(queue, latencyRowCount);
            };
            latencyRowCount[0] = latencyRowCount[0] + 1;
          };

          double[] latencyValue = (double[]) latencyValues.get(queue);
          if (latencyValue == null ) {
            latencyValue = new double[1];
            latencyValue[0] = 0.0;
            latencyValues.put(queue, latencyValue);
          };
          latencyValue[0] = latencyValue[0] + latency;
        }

      } catch( Exception e ) {
        System.out.println(e);
        e.printStackTrace();
      }

      return latestMonitorTime;
    }

    protected Timestamp getTPValues(Timestamp previousLatestMonitorTime, ResultSet rs, Hashtable tpValues, Hashtable latestTimes) {
      Timestamp latestMonitorTime = previousLatestMonitorTime;
      try {
        while (rs.next() ) {
          String queue = rs.getString(2);
          Timestamp  time = rs.getTimestamp(1);
          int rowsApplied = rs.getInt(3);
//        System.out.println("queue " + queue + " had rows: " + rowsApplied );

          if (time.after(latestMonitorTime) ) {
            latestMonitorTime = (Timestamp) time.clone();        // Save the latest timestamp for next WHERE clause
          };

          Timestamp latestTimeForQueue = (Timestamp) latestTimes.get(queue);
          if (latestTimeForQueue == null || time.after(latestTimeForQueue) ) {
            latestTimes.put(queue, time.clone() );
          }

          // Keep sum of total rows by queue
          double[] tpValue = (double[]) tpValues.get(queue);
          if (tpValue == null ) {
            tpValue = new double[1];
            tpValue[0] = 0.0;
            tpValues.put(queue, tpValue);
          };
          tpValue[0] = tpValue[0] + rowsApplied;
        }
      } catch( Exception e ) {
        System.out.println(e);
        e.printStackTrace();
      }

      return latestMonitorTime;
    }

   protected String[] getMonitorQueries() {
     String[] sqlStatements = new String[2];
     sqlStatements[0] = "SELECT  MONITOR_TIME, CURRENT_LOG_TIME FROM " +
                        schema+".IBMQREP_CAPMON  WHERE MONITOR_TIME> ? " +
                        " ORDER BY  MONITOR_TIME DESC" ;
     sqlStatements[1] = "SELECT  MONITOR_TIME, SENDQ, ROWS_PUBLISHED FROM " +
                        schema+".IBMQREP_CAPQMON  WHERE MONITOR_TIME> ? " +
                        " ORDER BY  MONITOR_TIME DESC" ;
     return sqlStatements;
   }

   protected String getRefreshIntervalQuery() {
     return  "SELECT MONITOR_INTERVAL, ARCH_LEVEL FROM " + schema + ".IBMQREP_CAPPARMS ";
   }

}