import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.*;
import java.sql.*;
import java.beans.*;
import java.text.*;

/**
 * This class is the basic visual panel that monitors a specific queue.
 *
 * @author tjacopi
 *
 */
public class QMonitorPanel extends JPanel  {

    protected static final ImageIcon TPTrendDecIcon = new ImageIcon("TPTrendDec.gif");
    protected static final ImageIcon TPTrendIncIcon = new ImageIcon("TPTrendInc.gif");
    protected static final ImageIcon LatencyTrendDecIcon = new ImageIcon("TrendDec.gif");
    protected static final ImageIcon LatencyTrendIncIcon = new ImageIcon("TrendInc.gif");
    protected static final ImageIcon TrendNeutralIcon    = new ImageIcon("TrendNut.gif");

    protected static final int TotalValues = 1000;
    protected static final int TrendValues = 5;
    protected static final double MaxLatencySeconds = 9999.9;
    protected static final double MaxTPRows         = 999999.9;
    protected Color[] latencySegmentColors =  null;
    protected String[] latencySegmentText = null;
    protected Color[] tpSegmentColors =  { Color.cyan };
    protected Color   tpExceptionColor =  Color.red;

    public    static final int LatencyGraph    = 1;
    public    static final int ThroughputGraph = 2;

    protected DataGatherer   dataGatherer = null;

    protected SegmentedData  latencyData = null;
    protected JPanel         latencyPanel = null;
    protected ScrollingGraph latencyGraph = null;
    protected AveragePanel   latencyAveragePanel = null;
    protected LegendPanel    latencyLegendPanel = null;
    protected String         latencyNoDataMsg   = null;

    protected SegmentedData  tpData = null;
    protected JPanel         tpPanel = null;
    protected ScrollingGraph tpGraph = null;
    protected AveragePanel   tpAveragePanel = null;
    protected LegendPanel    tpLegendPanel = null;

    protected String         queueName        = null;
    protected String         panelTitle       = null;
    protected boolean        hasSeparator = false;
    protected boolean        horizontalLayout = true;

    public QMonitorPanel(DataGatherer dataGatherer, String queueName,
                               Color[] latencyColors, String[] latencyText) {
      this.queueName       = queueName;
      this.dataGatherer = dataGatherer;
      latencySegmentColors = latencyColors;
      latencySegmentText = latencyText;
      setBackground(Color.black);

      setLayout( new BorderLayout() );
    }

    public void setTitle(String title) {
      panelTitle = title;
    }

    public String getTitle() {
      return panelTitle;
    }

    public void setHorizontalLayout(boolean isHorizontal) {
      if (isHorizontal != horizontalLayout) {                    // Insure we have a change
        horizontalLayout = isHorizontal;
        rebuildMainPanel();

        if (!horizontalLayout) {
          setBorder( new EmptyBorder(0, 0, 0, 10) );
        } else {
          setBorder( null );
        }
      };
    }

    public boolean isHorizontalLayout() {
      return horizontalLayout;
    }

    public void setLatencyNoDataMsg(String msg) {
      latencyNoDataMsg = msg;
    }

    public void setTPColors(Color segmentColor) {
      tpSegmentColors[0] = segmentColor;
    }

    public AveragePanel getAveragePanel(int graphType) {
      AveragePanel ap = null;
      if ( graphType == LatencyGraph ) {
        ap = latencyAveragePanel;
      } else if ( graphType == ThroughputGraph ) {
        ap = tpAveragePanel;
      };
      return ap;
    }

    public LegendPanel getLegendPanel(int graphType) {
      LegendPanel lp = null;
      if ( graphType == LatencyGraph ) {
        lp = latencyLegendPanel;
      } else if ( graphType == ThroughputGraph ) {
        lp = tpLegendPanel;
      };
      return lp;
    }

    public void reset() {
      if (latencyPanel != null) {
        latencyData.reset();
        latencyGraph.reset();
        latencyAveragePanel.reset();
      };

      if (tpPanel != null) {
        tpData.reset();
        tpGraph.reset();
        tpAveragePanel.reset();
      };

    }

    public boolean hasSeparator() {
      return hasSeparator;
    }

    public void setSeparator(boolean separator) {
      hasSeparator = separator;
      if (hasSeparator) {
        add( new BorderLine(), "South");
      } else {
        Component[] c = getComponents();
        for (int i=c.length-1; i>=0; i--) {
          if (c[i] instanceof BorderLine) {
            remove(c[i]);
          }
        }
      }
    }

    public boolean isShowing(int graphType) {
      boolean showing = false;
      if ( (graphType & LatencyGraph)>0 ) {
        if ( (graphType & ThroughputGraph)>0 ) {
          showing = (tpPanel != null) && (latencyPanel != null);
        } else {
          showing = (latencyPanel != null);
        }
      } else if ( (graphType & ThroughputGraph)>0 ) {
        showing = (tpPanel != null);
      };
      return showing;
    }

    public String getQueueName() {
      return queueName;
    }

    public void show(int graphType) {
      if ( !isShowing(graphType) ) {
        if ( (graphType & LatencyGraph)>0 ) {
          buildLatencyPanel();
        };
        if ( (graphType & ThroughputGraph)>0 ) {
          buildTPPanel();
        };
        rebuildMainPanel();
      };
    }

    public void hide(int graphType) {
      if ( isShowing(graphType) ) {
        if ( (graphType & LatencyGraph)>0 ) {
          latencyData = null;
          latencyPanel = null;
          latencyGraph = null;
          latencyAveragePanel = null;
          latencyLegendPanel = null;
        };
        if ( (graphType & ThroughputGraph)>0 ) {
          tpData = null;
          tpPanel = null;
          tpGraph = null;
          tpAveragePanel = null;
          tpLegendPanel = null;
        };

        rebuildMainPanel();
      };
    }

    protected void buildPanelTitle() {
      String queueNameText = panelTitle;

      if (queueNameText == null) {
        queueNameText = queueName;
      }
      JLabel queueNameLB =  new JLabel(queueNameText, SwingConstants.CENTER);
      queueNameLB.setForeground(Color.yellow);
      queueNameLB.setBackground(Color.black);
      queueNameLB.setOpaque(true);
      Font f = queueNameLB.getFont();
      Font f2 = f.deriveFont(f.getSize() + 2);
      Font f3 = f2.deriveFont(Font.BOLD);
      queueNameLB.setFont(f3);

      add( queueNameLB, "North");
    }

    protected void rebuildMainPanel() {
      removeAll();
      buildPanelTitle();

      JPanel mainPanel = new JPanel();
      mainPanel.setBackground(Color.black);
      int paneCount = 0;
      if (latencyPanel != null) {
        paneCount++;
      };
      if (tpPanel != null) {
        paneCount++;
      };
      if (paneCount >0){
        if (horizontalLayout) {
          mainPanel.setLayout( new GridLayout(1,paneCount) );
        } else {
          mainPanel.setLayout( new GridLayout(paneCount,1) );
        }
        if (latencyPanel != null) {
          mainPanel.add(latencyPanel);
        };
        if (tpPanel != null) {
          if (paneCount == 2 && horizontalLayout ) {
            tpPanel.setBorder( new EmptyBorder(0, 50, 0, 0) );
          } else {
            tpPanel.setBorder( null );
          }
          mainPanel.add(tpPanel);
        };
      }

      // If we are vertical, insure graph label space is the same
      if (!horizontalLayout && paneCount == 2) {
        int labelWidth = latencyGraph.getPreferredLabelWidth();
        int legendWidth = latencyLegendPanel.getPreferredSize().width;
        tpGraph.setLabelWidth(labelWidth + legendWidth );   // Do this so the graphs line up.
      };

      add(mainPanel, "Center");

      if (hasSeparator) {
        add( new BorderLine(), "South");
      };
      invalidate();
      validate();
    }

    protected void buildLatencyPanel() {
      latencyPanel = new JPanel();
      latencyPanel.setBackground(Color.black);
      latencyPanel.setLayout(new BorderLayout() );

      String propertyUpdateKey = dataGatherer.getLatencyPropertyKeyFor(queueName);

/*    // Setup a set of data values that will be sent if no data values are feteched from monitor tables
      double[] noLatencyDataValues = new double[latencySegmentColors.length];
      for (int i=0; i<latencySegmentColors.length; i++) {
        noLatencyDataValues[i] = 0.0;
      }
      latencyData = new SegmentedData(latencySegmentColors.length, TotalValues, propertyUpdateKey, noLatencyDataValues);
*/    latencyData = new SegmentedData(latencySegmentColors.length, TotalValues, propertyUpdateKey, null);
      dataGatherer.addPropertyChangeListener(latencyData);

      latencyGraph = new ScrollingGraph(latencySegmentColors, latencyData, "{0} sec", MaxLatencySeconds);
      latencyPanel.add(latencyGraph, "Center");

      String avgText = "Average latency {0} sec   Trend: ";
//    latencyAveragePanel = new AveragePanel(latencyData, avgText, LatencyTrendIncIcon, LatencyTrendDecIcon, TrendNeutralIcon, TrendValues, false);
      latencyAveragePanel = new AveragePanel(latencyData, avgText, LatencyTrendIncIcon, LatencyTrendDecIcon, TrendNeutralIcon, TrendValues, true, latencyNoDataMsg);
      latencyPanel.add(latencyAveragePanel, "South");

      if (latencySegmentText != null) {
        latencyLegendPanel = new LegendPanel(latencyData, "Latency", latencySegmentText, latencySegmentColors);
        latencyPanel.add(latencyLegendPanel,"East");
      };
    }

    protected void buildTPPanel() {
      tpPanel = new JPanel();
      tpPanel.setBackground(Color.black);
      tpPanel.setLayout(new BorderLayout() );

      String propertyUpdateKey = dataGatherer.getThroughputPropertyKeyFor(queueName);
/*    // Setup a set of data values that will be sent if no data values are feteched from monitor tables
      double[] noTPDataValues = new double[tpSegmentColors.length];
      for (int i=0; i<tpSegmentColors.length; i++) {
        noTPDataValues[i] = 0.0;
      }
      tpData = new SegmentedData(tpSegmentColors.length, TotalValues, propertyUpdateKey, noTPDataValues);
*/    tpData = new SegmentedData(tpSegmentColors.length, TotalValues, propertyUpdateKey, null);
      dataGatherer.addPropertyChangeListener(tpData);

      tpGraph = new ScrollingGraph(tpSegmentColors, tpData, "{0} rows/sec", MaxTPRows);
      tpGraph.displayNonZeroExtendedValuesAs(tpExceptionColor);
      tpPanel.add(tpGraph, "Center");

      // This is a hack.  The first choice to show "no activity" message is the latency panel.  If we do not
      // have one, then we will show it as part of the throughput panel.
      boolean needToShowNoActivityMsg = (latencyLegendPanel == null);
      String avgText = "Average throughput {0} rows/sec   Trend: ";
      tpAveragePanel = new AveragePanel(tpData, avgText, TPTrendIncIcon, TPTrendDecIcon, TrendNeutralIcon, TrendValues, needToShowNoActivityMsg, null);
      tpPanel.add(tpAveragePanel, "South");
    }
}