import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.sql.*;


/**
 * The action to stop the graphs.
 *
 * @author tjacopi
 *
 */
public class Utils {

  public static void showTrace(String sql, Timestamp lastTime, Connection conn, String title) {

    try {
      File tempFile = File.createTempFile("mon", ".htm");
      FileWriter fwriter = new FileWriter(tempFile);
      fwriter.write("<html> <head><title>");
      fwriter.write(title);
      fwriter.write("</title></head><body>");

      PreparedStatement stmt = conn.prepareStatement(sql);
      if (lastTime != null) {
        stmt.setTimestamp(1, lastTime );
      };
      ResultSet rs = stmt.executeQuery();
      int rowCount = format(rs, fwriter);
      rs.close();
      stmt.close();
      if (rowCount == 0) {
        fwriter.write("No messages available");
      };
      fwriter.write("</body></html>");
      fwriter.flush();
      fwriter.close();

      launchBrowser(tempFile.getAbsolutePath());
    } catch( Exception e ) {
      System.out.println(e);
      e.printStackTrace();
    }

  }

  public static void launchBrowser(String htmlFile) {

    try {
      String operatingSystem = System.getProperty( "os.name" ).toUpperCase();
      String cmd = null;
      if ( operatingSystem.equals( "WINDOWS 95" ) ||
           operatingSystem.equals( "WINDOWS ME" ) ||
           operatingSystem.equals( "WINDOWS 98" ) ) {
           cmd = " command.com /c start " + htmlFile;
      } else if (operatingSystem.indexOf("WINDOW")>=0 ) {
           cmd = "cmd.exe /c start " + htmlFile;
      };
      if (cmd != null) {
        Runtime runtime = Runtime.getRuntime();
        runtime.exec(cmd);
      };
    } catch( Exception e ) {
      System.out.println(e);
      e.printStackTrace();
    }

  }

  public static int  format(ResultSet rs, Writer writer) {
    int rowCount = 0;
    try {
      writer.write("<table border=\"1\">");
      writer.write("<tr><th>Msg Type</th> <th>Time</th> <th>Message Text</th></tr>");
      try {
        boolean bold = false;
        while (rs.next() ) {

          writer.write("<tr>");
          String msgType = rs.getString(1);
          Timestamp  time = rs.getTimestamp(2);
          String msg     = rs.getString(3);

          bold = "ERROR".equalsIgnoreCase(msgType.trim() );

          addCellValue(bold, msgType, writer);
          addCellValue(bold, time.toString(), writer);
          addCellValue(bold, msg, writer);
          writer.write("</tr>");
          rowCount++;
        }
      } catch( Exception e ) {
        System.out.println(e);
        e.printStackTrace();
      }

      writer.write("</table>");
    } catch( Exception e ) {
      System.out.println(e);
      e.printStackTrace();
    }

    return rowCount;
  }

  public static void showExceptions(String dbName, String schema, Connection conn, long fromMillis) {
    try {
      String title = dbName + "(" + schema + ")" + " QApply Exceptions";
      File tempFile = File.createTempFile("mon", ".htm");
      FileWriter fwriter = new FileWriter(tempFile);
      fwriter.write("<html> <head><title>");
      fwriter.write(title);
      fwriter.write("</title></head><body>");

       try {
         Class printExceptionClass = Class.forName("PrintExceptionTable");
         Class writerClass = Class.forName("java.io.Writer");
         Class stringClass = Class.forName("java.lang.String");
         Class connectionClass = Class.forName("java.sql.Connection");
         Class longClass = Class.forName("java.lang.Long");
         Class[] parmTypes = { writerClass, stringClass, connectionClass, longClass };
         Method method = printExceptionClass.getMethod("buildHtml", parmTypes);
         if (method != null) {
           Long fromMillisObj = null;
           if (fromMillis >0) {
             fromMillisObj = new Long(fromMillis);
           };
           Object[] parms = {fwriter, schema, conn, fromMillisObj };
           method.invoke(null, parms);                   // Will build the html
         } else {
           fwriter.write("Could not find buildHtml method in PrintExceptionTable class <br>");
         }
       } catch (Throwable t) {
         t.printStackTrace();
         fwriter.write("Error loading xml parser.  Insure that your CLASSPATH environment variable contains PrintExceptionTable.jar <br>");
         fwriter.write("Error is ");
         fwriter.write(t.toString());
         fwriter.write("<br><br>");
       }

      fwriter.write("</body></html>");
      fwriter.flush();
      fwriter.close();

      launchBrowser(tempFile.getAbsolutePath());
    } catch( Exception e ) {
      System.out.println(e);
      e.printStackTrace();
    }
  }

  private static void addCellValue(boolean bold, String text, Writer writer) throws IOException {
    writer.write("<td>");
    if (bold) {
      writer.write("<b>");
    };
    writer.write(text);
    if (bold) {
      writer.write("</b>");
    };
    writer.write("</td>");
  }
}